<?php
/*
Plugin Name: WP_Query Patch - Test Data
Plugin URI: http://piklist.com
Description: Test Data for the WP_Query Patch Plugin
Version: 0.0.1
Author: Kevin Miller
Author URI: http://piklist.com
*/

$wp_query_patch_test_data = new WP_Query_Patch_Test_Data();

class WP_Query_Patch_Test_Data {
    
  protected $post_type = 'post_test';

  protected $post_count = 10000;
  
  public function __construct() {
    add_action( 'init', array( $this, 'register_post_type' ) );
  }
  
  public function run_test_query() {
    $query = new WP_Query( array( 
      'post_type' => $this->post_type,
      'posts_per_page' => 10,
      'meta_query' => array( 
        'relation' => 'AND',
        array( 
          'relation' => 'OR',
          array( 
      			'key' => 'test_meta_0',
            'value' => 0,
      			'compare' => '>=',
            'type' => 'NUMERIC'
      		),
          array( 
      			'key' => 'test_meta_1',
            'value' => 0,
      			'compare' => '>=',
            'type' => 'NUMERIC'
      		)
        ),
        array( 
    			'key' => 'test_meta_0',
          'value' => 50,
    			'compare' => '>=',
          'type' => 'NUMERIC'
    		),
        array( 
    			'key' => 'test_meta_1',
          'value' => 50,
    			'compare' => '>=',
          'type' => 'NUMERIC'
    		),
        array( 
    			'key' => 'test_meta_2',
          'value' => 50,
    			'compare' => '>=',
          'type' => 'NUMERIC'
    		),
        array( 
    			'key' => 'test_meta_3',
          'value' => 50,
    			'compare' => '>=',
          'type' => 'NUMERIC'
    		),
        array( 
    			'key' => 'test_meta_4',
          'value' => 50,
    			'compare' => '>=',
          'type' => 'NUMERIC'
    		),
        array( 
    			'key' => 'test_meta_5',
          'value' => 50,
    			'compare' => '>=',
          'type' => 'NUMERIC'
    		),
        array( 
    			'key' => 'test_meta_6',
          'value' => 50,
    			'compare' => '>=',
          'type' => 'NUMERIC'
    		),
        array( 
    			'key' => 'test_meta_7',
          'value' => 50,
    			'compare' => '>=',
          'type' => 'NUMERIC'
    		),
        array( 
    			'key' => 'test_meta_8',
          'value' => 50,
    			'compare' => '>=',
          'type' => 'NUMERIC'
    		),
        array( 
    			'key' => 'test_meta_9',
          'value' => 50,
    			'compare' => '>=',
          'type' => 'NUMERIC'
    		),
        array( 
    			'key' => 'test_meta_10',
          'value' => 50,
    			'compare' => '>=',
          'type' => 'NUMERIC'
    		)
      )
    ) );
  }
  
  public function register_post_type() {
    global $pagenow;
    
  	register_post_type( $this->post_type, array( 
  		'public' => true,
  		'publicly_queryable' => true,
      'label' => __( 'Test Posts' ),
  		'show_ui' => true,
  		'show_in_menu' => true,
  		'query_var' => true,
  		'rewrite'=> array( 
        'slug' => 'test-post'
      ),
  		'capability_type' => 'post',
  		'has_archive' => true,
  		'hierarchical' => false,
  		'menu_position' => null,
  		'supports' => array(  
        'title',
        'editor',
        'custom-fields',
        'excerpt'
      )
  	) );
    
    if ( $pagenow == 'edit.php' && ( isset( $_REQUEST['post_type'] ) && $_REQUEST['post_type'] == $this->post_type ) ) {
      $this->performance();
      
      if ( wp_count_posts( $this->post_type )->publish <= $this->post_count ) {
        $this->load_data();
      }
      
      $this->run_test_query();
    }
  }
  
  public function load_data() {
    global $wpdb;
  
    for ( $i = 0; $i <= $this->post_count; $i++ ) {
      $post = array( 
        'post' => array( 
          'post_type' => 'post_test',
          'post_title' => __( 'Test Post #' . $i ),
          'post_content' => 'Lorem ipsum dolor sit amet, consectetur adipiscing elit. Curabitur et augue porta, efficitur erat sed, facilisis ex. In quis sollicitudin nisi, sed imperdiet libero. Ut blandit finibus suscipit. Duis nisl ipsum, malesuada in tincidunt quis, lacinia scelerisque lorem. Sed et ligula consectetur lectus pretium ultrices sed vitae elit. Quisque iaculis dolor vel feugiat mollis. Donec pulvinar finibus iaculis. Vestibulum sed pulvinar risus. Cras vitae odio ut mi consectetur tincidunt tristique et nisl. Sed finibus efficitur nisi, tempor rhoncus turpis luctus tristique. Vivamus id sapien quis risus eleifend maximus. Quisque sed pellentesque massa. In viverra, dolor sed rhoncus pharetra, magna eros venenatis ante, sed tempus arcu nisi et leo.',
          'post_excerpt' => 'Lorem ipsum dolor sit amet, consectetur adipiscing elit. Curabitur et augue porta, efficitur erat sed, facilisis ex. In quis sollicitudin nisi, sed imperdiet libero. Ut blandit finibus suscipit.',
          'post_name' => sanitize_title( 'Test Post #' . $i ),
          'post_status' => 'publish'
        ),
        'meta' => array()      
      );
  
      for ( $j = 0; $j <= 10; $j++ ) {
        $post['meta']['test_meta_' . $j] = rand( 5, 100 );
      }
  
      $found = $wpdb->get_var( $wpdb->prepare( "SELECT ID FROM $wpdb->posts WHERE post_name = %s AND post_type = %s", $post['post']['post_name'], $this->post_type ) );

      if ( $found ) {
        $post['post']['ID'] = $found;
        $post['post']['ID'] = wp_update_post( $post['post'] );
      } else {
        $post['post']['ID'] = wp_insert_post( $post['post'] );
      }

      foreach ( $post['meta'] as $meta_key => $meta_value ) {
        delete_post_meta( $post['post']['ID'], $meta_key );
        add_post_meta( $post['post']['ID'], $meta_key, $meta_value, true );
      }
    }
  }
  
  public function performance() {
    if ( !ini_get( 'safe_mode' ) ) { 
      ini_set( 'max_execution_time', -1 );
      ini_set( 'memory_limit', -1 );
    }
  }
}