<?php
/*
Plugin Name: WP_Query Patch
Plugin URI: http://piklist.com
Description: Test a possible WP_Query Patch. Install <a href="https://wordpress.org/plugins/debug-bar/">Debug Bar</a> with define( 'SAVEQUERIES', true ); in the wp-config.php. Ticket <a href="https://core.trac.wordpress.org/ticket/30044">https://core.trac.wordpress.org/ticket/30044</a>
Version: 0.0.1
Author: Kevin Miller
Author URI: http://piklist.com
*/

$wp_query_patch = new WP_Query_Patch();

class WP_Query_Patch {
  
  protected $primary_table;

  protected $primary_id_column;

  protected $meta_table;

  protected $meta_id_column;
  
  public function __construct() {
    add_filter( 'get_meta_sql', array( $this, 'get_meta_sql' ), 101, 6 );
  }
  
  public function get_meta_sql( $sql, $query, $type, $primary_table, $primary_id_column, $context, $parent_relation = 'AND', $depth = 0 ) {
    $this->primary_table = $primary_table;
    $this->primary_id_column = $primary_id_column;
    $this->meta_table = substr( $primary_table, 0, strlen( $primary_table ) - 1 ) . 'meta';;
    $this->meta_id_column = $type . '_id';
    
    return $this->get_sql_for_query( $query, 0 );
  }
  
  
  
  /* --------------------------------------------------------------------------------------------------------------
    Proposed patch - replacement for get_sql_for_query in wp-includes/meta.php
  
    TO PATCH wp-includes/meta.php
      - switch lines at end of function below (see comment) and replace get_sql_for_query
      - remove get_sql_for_clause from wp-includes/meta.php
  -------------------------------------------------------------------------------------------------------------- */
  
  public function get_sql_for_query( $query, $depth = 0, $parent_relation = 'AND' )
  {
    global $wpdb;

    $sql_chunks = array( 
      'join'  => array(),
      'where' => array()
    );

    $sql = array( 
      'join'  => '',
      'where' => ''
    );

    $sql_nested = false;

    $indent = '';
    for ( $i = 0; $i < $depth; $i++ ) {
      $indent .= "  ";
    }
    
    $relation = 'AND';
    foreach ( $query as $key => $clause ) {
      if ( 'relation' === $key ) {
        $relation = $query['relation'];
      } 
    }
    
    $primary_ids = array();
    foreach ( $query as $key => $clause ) {
      if ( is_array( $clause ) ) {
        if ( $this->is_first_order_clause( $clause ) ) {
          if ( isset( $clause['compare'] ) ) {
            $meta_compare = strtoupper( $clause['compare'] );
          } else {
            $meta_compare = isset( $clause['value'] ) && is_array( $clause['value'] ) ? 'IN' : '=';
          }

          if ( ! in_array( $meta_compare, array( 
            '=', '!=', '>', '>=', '<', '<=',
            'LIKE', 'NOT LIKE',
            'IN', 'NOT IN',
            'BETWEEN', 'NOT BETWEEN',
            'EXISTS', 'NOT EXISTS',
            'REGEXP', 'NOT REGEXP', 'RLIKE'
           ) ) ) {
            $clause['compare'] = '=';
          }

          $meta_compare = $clause['compare'];

          $meta_key_string = '';
          if ( array_key_exists( 'key', $clause ) ) {
            if ( 'NOT EXISTS' === $meta_compare ) {
              $meta_key_string = $this->meta_id_column . ' IS NULL';
            } else {
              $meta_key_string = $wpdb->prepare( "meta_key = %s", trim( $clause['key'] ) );
            }
          }

          $meta_value_string = '';
          if ( array_key_exists( 'value', $clause ) ) {
            $meta_value = $clause['value'];
            $meta_type = $this->get_cast_for_type( isset( $clause['type'] ) ? $clause['type'] : '' );

            if ( in_array( $meta_compare, array( 'IN', 'NOT IN', 'BETWEEN', 'NOT BETWEEN' ) ) ) {
              if ( ! is_array( $meta_value ) ) {
                $meta_value = preg_split( '/[,\s]+/', $meta_value );
              }
            } else {
              $meta_value = trim( $meta_value );
            }

            if ( 'IN' == substr( $meta_compare, -2 ) ) {
              $meta_value_string = '( ' . substr( str_repeat( ',%s', count( $meta_value ) ), 1 ) . ' )';
            } elseif ( 'BETWEEN' == substr( $meta_compare, -7 ) ) {
              $meta_value = array_slice( $meta_value, 0, 2 );
              $meta_value_string = '%s AND %s';
            } elseif ( 'LIKE' == $meta_compare || 'NOT LIKE' == $meta_compare ) {
              $meta_value = '%' . $wpdb->esc_like( $meta_value ) . '%';
              $meta_value_string = '%s';
            } else {
              $meta_value_string = '%s';
            }

            $meta_value_string = $wpdb->prepare( "CAST( meta_value AS {$meta_type} ) {$meta_compare} {$meta_value_string}", $meta_value );
          }

          if ( $meta_key_string != '' ) {
            $meta_compare_string = $meta_key_string . ' AND ' . $meta_value_string;
          }

          if ( 'AND' == $relation && ! empty( $primary_ids ) ) {
            $meta_compare_string .= " $relation $this->meta_id_column IN ( " . implode( ',', $primary_ids ) . " )";
          }

          if ( $_primary_ids = $wpdb->get_col( "SELECT $this->meta_id_column FROM $this->meta_table WHERE $meta_compare_string GROUP BY $this->meta_id_column" ) ) {
            $primary_ids = ( 'AND' == $relation && ! empty( $primary_ids ) ) ? array_intersect( $primary_ids, $_primary_ids ) : array_merge( $primary_ids, $_primary_ids );
          } elseif ( 'AND' == $relation ) {
            $primary_ids = array();
            break;
          }
        } else {
          $sql_nested = $depth == 0;

          $clause_sql = $this->get_sql_for_query( $clause, $depth + 1, $relation );
          
          $sql_chunks['where'][] = $clause_sql['where'];
          $sql_chunks['join'][]  = $clause_sql['join'];
        }
      }
    }

    if ( ! empty( $primary_ids ) ) {
      $sql_chunks['where'][] = " $this->primary_table.$this->primary_id_column IN ( " . implode( ',', $primary_ids ) . " ) ";
      
      if ( ! empty( $sql_chunks['where'] ) ) {
        // NOTE: Use commented line for patch
        // $sql['where'] =  ( $sql_nested ? "( " : null ) . "\n$indent" . implode( "\n $indent $relation \n" . $indent, $sql_chunks['where'] ) . "\n$indent" . ( $sql_nested ? " )" : null );
        $sql['where'] =  ( $depth == 0 ? " $parent_relation " : null ) . ( $sql_nested ? "( " : null ) . "\n$indent" . implode( "\n $indent $relation \n" . $indent, $sql_chunks['where'] ) . "\n$indent" . ( $sql_nested ? " )" : null );
      }
    }

    return $sql;
  }
  





  
  /* --------------------------------------------------------------------------------------------------------------
    Pulled from wp-includes/meta.php with no changes
  -------------------------------------------------------------------------------------------------------------- */
  
	protected function is_first_order_clause( $query ) {
		return isset( $query['key'] ) || isset( $query['value'] );
	}

	protected function get_cast_for_type( $type = '' ) {
		if ( empty( $type ) ) {
      return 'CHAR';
    }
    
		$meta_type = strtoupper( $type );

		if ( ! preg_match( '/^( ?:BINARY|CHAR|DATE|DATETIME|SIGNED|UNSIGNED|TIME|NUMERIC(?:\(\d+(?:,\s?\d+ )?\))?|DECIMAL(?:\(\d+(?:,\s?\d+)?\))?)$/', $meta_type ) ) {
      return 'CHAR';
    }
    
		if ( 'NUMERIC' == $meta_type ) {
      $meta_type = 'SIGNED';
    }
    
		return $meta_type;
	}
}